# NAME:
#	mktime - generate secs since epoch for given date
#
# SYNOPSIS:
#	$secs = &mktime($date, $tz);
#
# DESCRIPTION:
#	This function is a bit like the ANSI C function of the same
#	name in that it returns the seconds since the UNIX epoch for
#	a given date.  Unlike mktime(3), this routine does not
#	manipulate a 'tm struct' but instead accepts a date string
#	like "93-09-28 23:30:00" actually any format as long after
#	striping all non-numerics is in YYMMDDHHMISS format will do.
#	You can omit any trailing part (ie. YYMMDD will work).
#
#	"$Tz" is the correction for local time expressed as minutes
#	west of GMT.  If "$tz" is omited we work it out.  If it is
#	presented as hours[:min] west of GMT and we will handle that
#	too. 
#
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

#
# RCSid:
#	$Id: mktime.pl,v 1.7 2002/05/27 00:40:52 sjg Exp $
#	
#
#	@(#)Copyright (c) 1991, Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

sub isleap {
  local($year) = @_;
  
  (((($year) % 4) == 0 && (($year) % 100) != 0) || (($year) % 400) == 0);
}

sub mktime {
    local($date,$tz) = @_;
    local($year,$month,$day,$hour,$min,$sec);
    local($i,$years,$leaps,$secs,@ltime);
    
    if ($tz eq '') {
      if ($_TZ eq '') {
	# this logic is from perl5's timelocal.pl
	local(@epoch) = localtime(0);
	$_TZ = $epoch[2] * 60 + $epoch[1];
	if ($_TZ > 0) {
	  $_TZ = 1440 - $_TZ;
	  $_TZ -= 1440 if ($epoch[5] == 70);
	}

      }
      $tz = $_TZ;
    } elsif ($tz =~ m/^([^:]+):*(.*)/) {
      $hour = $1;
      $min = $2;
      if ($hour < 23 && $hour > -23) {
	$tz = $hour * 60 + $min;
      }
    }
    if (!defined(@Dayspermonth)) {
      # first time initialization
      @Dayspermonth = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
      @ltime = localtime;
      $mktime_pivot = ($ltime[5] + 30) % 100;
    }
    if (!defined(%MonthNumber)) {
      %MonthNumber = ('Jan', '01',
		      'Feb', '02',
		      'Mar', '03',
		      'Apr', '04',
		      'May', '05',
		      'Jun', '06',
		      'Jul', '07',
		      'Aug', '08',
		      'Sep', '09',
		      'Oct', '10',
		      'Nov', '11',
		      'Dec', '12',
		      );
    }
    if ($date =~ m/(([a-z]{3,3})[a-z]*)/io) {
      local ($mk, $m, $mn);
      
      $m = $1;
      $mk = $2;
      if (($mn = $MonthNumber{$mk}) ne '') {
	$date =~ s/$m/$mn/g;
      }
    }
    $date =~ s,[-/.: ],,g;
    ($year,$month,$day,$hour,$min,$sec) = unpack("A4A2A2A2A2A2", $date);
    if ($year > 6900 || $year < 1900 || $month < 1 || $month > 12) {
	($year,$month,$day,$hour,$min,$sec) = unpack("A2A2A2A2A2A2", $date);
	$year += ($year > $mktime_pivot) ? 1900 : 2000;
    }
    $years = $year - 1970;	# years since epoch
    $secs = $years * 365;	# days actually
    if ($year < 1972) {
      $leaps = 0;
    } else {
      $leaps = $year - 1972;	# first leap year since epoch
      $leaps = int($leaps / 4) + 1; # 1972 was a leap year.
    
      if (&isleap($year) && $month < 3) {
	$leaps--;			# already counted it.
      }
    }
    $secs += $leaps;		# still days
    for ($i = 0; $i < $month - 1; $i++) {
	$secs += $Dayspermonth[$i];
    }
    $secs += $day - 1 if ($day ne '');
    $secs *= 24;		# hours now
    $secs += $hour if ($hour ne '');
    $secs *= 3600;		# secs at last
    $secs += $tz * 60;		# time zone (minutes west of GMT)
    $secs += $min * 60 if ($min ne '');
    $secs += $sec if ($sec ne '');
#    print STDERR "year=$year,month=$month,day=$day,hour=$hour,min=$min,sec=$sec,secs=$secs\n";

    if ($tz eq $_TZ) {
      # we are using our timezone so this is meaningful.
      # check for daylight savings - at the time in question!
      local(@ltime) = localtime($secs);
      $secs -= 3600 if ($hour != $ltime[2]);
    }
    $secs;
}

1;
